unit ChooseMdlUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids, MathDef, Spin, Misc, v_lib;

type
  TChooseMdlParamsDialog = class(TForm)
    LBModels: TListBox;
    RGModelType: TRadioGroup;
    OkButton: TButton;
    CancelButton: TButton;
    GBModel: TGroupBox;
    GBParameters: TGroupBox;
    SGParameters: TStringGrid;
    GBLines: TGroupBox;
    SELines: TSpinEdit;
    GBSplit: TGroupBox;
    SESplit: TSpinEdit;
    GroupBox1: TGroupBox;
    EAx: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    EAy: TEdit;
    EAz: TEdit;
    EBx: TEdit;
    EBy: TEdit;
    EBz: TEdit;
    ECx: TEdit;
    ECy: TEdit;
    ECz: TEdit;
    CBScalesOnly: TCheckBox;
    procedure RGModelTypeClick(Sender: TObject);
    procedure LBModelsClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SELinesExit(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SESplitExit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    e_abc     : array[0..2,0..2] of TEdit;
    function  ValidateABC:Boolean;
  public
    lines     : integer;
    split     : integer;
    mdl       : PModel;
    abc       : array[0..2] of vec3_t;
    B         : array[0..511] of double;
    procedure FillLBModels;
  end;

var
  ChooseMdlParamsDialog: TChooseMdlParamsDialog;

procedure spatial(var a1,a2,a3,res:double);stdcall;

var
  progress : integer = 0;

implementation

uses MainUnit;

{$R *.DFM}

{$OPTIMIZATION OFF}

var
  args      : array[0..2] of double;
  y         : double;

procedure spatial(var a1,a2,a3,res:double);stdcall;
begin
  args[0]:=a1;
  args[1]:=a2;
  args[2]:=a3;
  ChooseMdlParamsDialog.mdl^.Value(@args,@ChooseMdlParamsDialog.B,y);
  res:=y;
  MainForm.StatusBar.SimpleText:=Format(' %d%%',[progress div 51]);
  progress:=progress+1;
end;

{$OPTIMIZATION ON}

procedure TChooseMdlParamsDialog.FillLBModels;
var
  i : integer;
  n : string;
begin
  if LBModels.Items.Count > 1 then exit;
  LBModels.Clear;
  case RGModelType.ItemIndex of
    0: begin
         for i:=0 to MainForm.Modules.ModelCount-1 do begin
           n:=MainForm.Modules.Models[i].Description;
           // there are probably better ways to determine model's type ...
           if Pos('',n) <> 0 then
             LBModels.Items.Add(n);
         end;
       end;
    1: begin
         for i:=0 to MainForm.Modules.ModelCount-1 do begin
           n:=MainForm.Modules.Models[i].Description;
           if MainForm.Modules.Models[i].RequiredDimension = 3 then
             LBModels.Items.Add(n);
         end;
       end;
    2: begin
         for i:=0 to MainForm.Modules.ModelCount-1 do begin
           n:=MainForm.Modules.Models[i].Description;
           LBModels.Items.Add(n);
         end;
       end;
  end;
  OkButton.Enabled := LBModels.Items.Count > 0;
  if OkButton.Enabled then
    LBModels.ItemIndex := 0
  else
    LBModels.ItemIndex := -1;
  LBModelsClick(nil);
end;

function  TChooseMdlParamsDialog.ValidateABC:Boolean;
var
  i,j:integer;
begin
  for i:=0 to 2 do begin
    for j:=0 to 2 do begin
      if not ValidateFloatEdit(e_abc[i][j],Self,@(abc[i][j])) then begin
        Result:=False;
        exit;
      end;
    end;
  end;
  Result:=True;
end;

procedure TChooseMdlParamsDialog.RGModelTypeClick(Sender: TObject);
begin
  FillLBModels;
end;

procedure TChooseMdlParamsDialog.LBModelsClick(Sender: TObject);
var
  i   : integer;
  m   : integer;
  OldCount : integer;
begin
  if OkButton.Enabled then begin
    m:=MainForm.Modules.ModelByName(LBModels.Items[LBModels.ItemIndex]);
    mdl:=MainForm.Modules.Models[m];
    OldCount:=SGParameters.ColCount;
    SGParameters.ColCount:=mdl^.AddenCount;
    for i:=0 to SGParameters.ColCount-1 do
      SGParameters.Cells[i,0]:=mdl^.Addens(i)^.StdBName;
    for i:=OldCount to SGParameters.ColCount-1 do
      SGParameters.Cells[i,1]:='0,0';
  end;
end;

procedure TChooseMdlParamsDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  i : integer;
begin
  if ModalResult <> mrOk then exit;
  i:=0;
  try
    while i<mdl^.AddenCount do begin
      B[i]:=StrToFloat(SGParameters.Cells[i,1]);
      i:=i+1;
    end;
    (*
    if z = mdl^.AddenCount then
    begin
      ShowMessage('  .    ,    ?');
      CanClose:=False;
    end;
    *)
  except
    SGParameters.Col:=i;
    SGParameters.Row:=1;
    ShowMessage('     !');
    CanClose:=False;
  end;
  CanClose:=ValidateABC;
  if not CanClose then
    ShowMessage('     !');
end;

procedure TChooseMdlParamsDialog.SELinesExit(Sender: TObject);
begin
  if not ValidateSpinEdit(Sender,Self) then
    SELines.Value:=5;
  lines:=SELines.Value;
end;

procedure TChooseMdlParamsDialog.SESplitExit(Sender: TObject);
begin
  if not ValidateSpinEdit(Sender,Self) then
    SESplit.Value:=4;
  split:=SESplit.Value
end;

procedure TChooseMdlParamsDialog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SELinesExit(nil);
  SESplitExit(nil);
end;

procedure TChooseMdlParamsDialog.FormCreate(Sender: TObject);
begin
  SGParameters.Cells[0,1]:='0,0';
  FillLBModels;
  e_abc[0][0]:=EAx;
  e_abc[0][1]:=EAy;
  e_abc[0][2]:=EAz;
  e_abc[1][0]:=EBx;
  e_abc[1][1]:=EBy;
  e_abc[1][2]:=EBz;
  e_abc[2][0]:=ECx;
  e_abc[2][1]:=ECy;
  e_abc[2][2]:=ECz;
  ValidateABC;
  lines:=SELines.Value;
  split:=SESplit.Value
end;

end.

