unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, ToolWin, ExtCtrls, MathDef, FormatDlg, v_lib, StdCtrls, Math;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    AppendItem: TMenuItem;
    ExitItem: TMenuItem;
    ContourMenu: TMenuItem;
    PlotItem: TMenuItem;
    ExportItem: TMenuItem;
    HelpMenu: TMenuItem;
    AboutItem: TMenuItem;
    StatusBar: TStatusBar;
    AppWorkSpace: TScrollBox;
    OutPanel: TPaintBox;
    FormatItem: TMenuItem;
    EditMenu: TMenuItem;
    CopyItem: TMenuItem;
    ClearItem: TMenuItem;
    SaveDialog: TSaveDialog;
    ViewMenu: TMenuItem;
    ConsoleItem: TMenuItem;
    ConSplitter: TSplitter;
    EConsole: TMemo;
    N1: TMenuItem;
    BatchItem: TMenuItem;
    procedure ExitItemClick(Sender: TObject);
    procedure AboutItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AppendItemClick(Sender: TObject);
    procedure PlotItemClick(Sender: TObject);
    procedure OutPanelPaint(Sender: TObject);
    procedure CopyItemClick(Sender: TObject);
    procedure ExportItemClick(Sender: TObject);
    procedure FormatItemClick(Sender: TObject);
    procedure EditMenuClick(Sender: TObject);
    procedure ContourMenuClick(Sender: TObject);
    procedure ClearItemClick(Sender: TObject);
    procedure ConsoleItemClick(Sender: TObject);
    procedure EConsoleKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EConsoleKeyPress(Sender: TObject; var Key: Char);
    procedure BatchItemClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    profile   : patchprofile_p;
    Bitmap    : TBitmap;
    Metafile  : TMetafile;
    procedure ExecuteBatch(BatchFileName : string);
  public
    Modules   : TMathDefinition;
    function  ConsoleCommand(cmd:string):Boolean;
  end;

var
  MainForm: TMainForm;

implementation

uses AppendMdlUnit, AboutUnit, ChooseMdlUnit, PPDraw;

{$R *.DFM}

function TMainForm.ConsoleCommand;
var
  pcmd : integer;
begin
  Result:=True;
  if copy(cmd,1,1) = '!' then begin
    WinExec(pchar(copy(cmd,2,length(cmd)-1)),SW_SHOWNORMAL);
  end else
  if (cmd = 'quit') or (cmd = 'exit') then begin
    ExitItemClick(Self);
  end else
  if cmd = 'clear' then begin
    EConsole.Clear;
  end else
  if cmd = 'helpabout' then begin
    AboutItemClick(Self);
  end else
  if cmd = 'append' then begin
    AppendItemClick(Self);
  end else
  if cmd = 'plot' then begin
    PlotItemClick(Self);
  end else
  if cmd = 'format' then begin
    FormatItemClick(Self);
  end else
  if cmd = 'export' then begin
    ExportItemClick(Self);
  end else
  if Copy(cmd,1,5) = 'batch' then begin
    if Length(cmd) = 5 then
      BatchItemClick(Self)
    else
    begin
      for pcmd := 6 to Length(cmd) do
        if Ord(Copy(cmd,pcmd,1)[1]) > 32 then
          break;
      if pcmd <= Length(cmd) then
        ExecuteBatch(Copy(cmd,pcmd,Length(cmd)-pcmd+1))
      else
        con_lputs(LOG_ERROR,'"batch" keyword must be followed by file name !'+#13);
    end;
  end else
    Result:=False;
end;

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.AboutItemClick(Sender: TObject);
begin
  AboutDialog.ShowModal;
end;

procedure ConOut(str:pchar);stdcall;
var
  s       : string;
  ch      : char;
  i       : integer;
begin
  s:='';
  i:=0;
  while true do begin
    ch:=char(u_8_array_p(str)^[i]);
    i:=i+1;
    if (ch = #0) or (ch = #$0D) or (ch = #$0A) then break;
    s:=s+ch;
  end;
  MainForm.EConsole.Lines.Add('>'+s);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Modules:=TMathDefinition.Create;
  Modules.FindModules('.');
  Bitmap:=TBitmap.Create;
  Bitmap.PixelFormat:=pfDevice;
  Bitmap.Height:=OutPanel.Height;
  Bitmap.Width:=OutPanel.Width;
  PatBlt(Bitmap.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,WHITENESS);
  Metafile:=TMetafile.Create;
  Metafile.Enhanced:=False;
  conAddCallback(@ConOut);
  conUpperLevel(LOG_INFO);
  profile := nil;
  SendMessage(EConsole.Handle,WM_CHAR,ord('>'),0);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  conRemoveCallback(@ConOut);
  Modules.Free;
  Bitmap.Free;
  Metafile.Free;
  if profile <> nil then
    ppFree(profile);
end;

procedure TMainForm.AppendItemClick(Sender: TObject);
var
  oldnum : integer;
begin
  if AppendMdlDialog.ShowModal = mrOk then
  begin
    oldnum:=Modules.ModelCount;
    Modules.FindModules(AppendMdlDialog.DirectoryListBox.Directory);
    ChooseMdlParamsDialog.FillLBModels;
    ShowMessageFmt(' %d .',[Modules.ModelCount-oldnum]);
  end;
end;

procedure TMainForm.PlotItemClick(Sender: TObject);
var
  center : vec3_t;
  R,a : vec_t;
begin
  if ChooseMdlParamsDialog.ShowModal = mrOk then
  begin
    if profile <> nil then
      ppFree(profile);
    profile:=nil;
    if Bitmap.Width <> FormatDialog.SEViewWidth.Value then
      Bitmap.Width:=FormatDialog.SEViewWidth.Value;
    if Bitmap.Height <> FormatDialog.SEViewHeight.Value then
      Bitmap.Height:=FormatDialog.SEViewHeight.Value;
    if OutPanel.Width <> FormatDialog.SEViewWidth.Value then
      OutPanel.Width:=FormatDialog.SEViewWidth.Value;
    if OutPanel.Height <> FormatDialog.SEViewHeight.Value then
      OutPanel.Height:=FormatDialog.SEViewHeight.Value;
    R:=Bitmap.Width/2; ///(2*sqr(cos(Pi/6)));
    center[0]:=Bitmap.Width/2;
    center[1]:=Bitmap.Height/2+Bitmap.Height/10;
    center[2]:=0;
    ChooseMdlUnit.progress:=0;
    a:=-Pi/2;
    if ChooseMdlParamsDialog.CBScalesOnly.Checked then
      profile:=ppCreateTriangular(center,R,a,nil,@spatial,ChooseMdlParamsDialog.lines,0)
    else
      profile:=ppCreateTriangular(center,R,a,@ChooseMdlParamsDialog.abc,@spatial,ChooseMdlParamsDialog.lines,0);
    StatusBar.SimpleText:='';
    if profile = nil then begin
      ShowMessage('    .   .');
      PatBlt(Bitmap.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,WHITENESS);
      BitBlt(OutPanel.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,Bitmap.Canvas.Handle,0,0,SRCCOPY);
    end
    else
    begin
      DrawPPOnCanvas(profile,Bitmap.Canvas,Bitmap.Width,Bitmap.Height,ChooseMdlParamsDialog.split);
      OutPanelPaint(Self);
    end;
  end;
end;

procedure TMainForm.OutPanelPaint(Sender: TObject);
begin
  if OutPanel.Visible then
  begin
    Windows.BitBlt(OutPanel.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,Bitmap.Canvas.Handle,0,0,SRCCOPY);
  end;
end;

procedure TMainForm.CopyItemClick(Sender: TObject);
begin
  (* Bitmap.SaveToClipboardFormat(CF_METAFILE, *)
end;

procedure TMainForm.ExportItemClick(Sender: TObject);
var
  MetaCanvas : TMetafileCanvas;
  {
  center : vec3_t;
  R,a : vec_t;
  dupe : patchprofile_p;
  w,h : integer;
  }
begin
  if (profile = nil) or (not SaveDialog.Execute) then exit;
  {
  Metafile.MMWidth:=FormatDialog.SEPageWidth.Value*100;
  Metafile.MMHeight:=FormatDialog.SEPageHeight.Value*100;
  }
  Metafile.Width:=Bitmap.Width;
  Metafile.Height:=Bitmap.Height;
  MetaCanvas:=TMetafileCanvas.Create(Metafile,0);
  {
  SetMapMode(MetaCanvas.Handle,MM_ANISOTROPIC);
  w:=round(FormatDialog.SEPageWidth.Value/100*256);
  h:=round(FormatDialog.SEPageHeight.Value/100*256);
  SetWindowExtEx(MetaCanvas.Handle,w,h,nil);
  SetViewportExtEx(MetaCanvas.Handle,w,h,nil);
  R:=w/2;
  center[0]:=w/2;
  center[1]:=h/2+h/10;
  center[2]:=0;
  ChooseMdlUnit.progress:=0;
  a:=-Pi/2;
  dupe:=ppCreateTriangular(center,R,a,@ChooseMdlParamsDialog.abc,@spatial,ChooseMdlParamsDialog.lines,0);
  StatusBar.SimpleText:='';
  }
  DrawPPOnCanvas(profile,MetaCanvas,Bitmap.Width,Bitmap.Height,ChooseMdlParamsDialog.split);
  MetaCanvas.Free;
  Metafile.SaveToFile(SaveDialog.FileName);
end;

procedure TMainForm.FormatItemClick(Sender: TObject);
begin
  FormatDialog.SEViewWidth.Enabled:=profile=nil;
  FormatDialog.SEViewHeight.Enabled:=profile=nil;
  if FormatDialog.ShowModal = mrOk then begin
    if OutPanel.Width <> FormatDialog.SEViewWidth.Value then
      OutPanel.Width:=FormatDialog.SEViewWidth.Value;
    if OutPanel.Height <> FormatDialog.SEViewHeight.Value then
      OutPanel.Height:=FormatDialog.SEViewHeight.Value;
    if profile <> nil then begin
      DrawPPOnCanvas(profile,Bitmap.Canvas,Bitmap.Width,Bitmap.Height,ChooseMdlParamsDialog.split);
      OutPanelPaint(Self);
    end;
  end;
end;

procedure TMainForm.EditMenuClick(Sender: TObject);
begin
  CopyItem.Enabled:=profile<>nil;
  ClearItem.Enabled:=profile<>nil;
end;

procedure TMainForm.ContourMenuClick(Sender: TObject);
begin
  PlotItem.Enabled:=Modules.AddenCount > 0;
  ExportItem.Enabled:=profile<>nil;
end;

procedure TMainForm.ClearItemClick(Sender: TObject);
begin
  ppFree(profile);
  profile:=nil;
  PatBlt(Bitmap.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,WHITENESS);
  BitBlt(OutPanel.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,Bitmap.Canvas.Handle,0,0,SRCCOPY);
end;

procedure TMainForm.ConsoleItemClick(Sender: TObject);
begin
  ConsoleItem.Checked:=not ConsoleItem.Checked;
  if ConsoleItem.Checked then begin
    ConSplitter.Visible:=True;
    EConsole.Visible:=True;
    ActiveControl:=EConsole;
  end else begin
    EConsole.Visible:=False;
    ConSplitter.Visible:=False;
  end;
end;

procedure TMainForm.EConsoleKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  cmd : string;
begin
  case Key of
    VK_RETURN:
    begin
      cmd:=EConsole.Lines[EConsole.Lines.Count-1];
      cmd:=Copy(cmd,2,length(cmd)-1);
      EConsole.Lines.Add('');
      if cmd <> '' then begin
        if not ConsoleCommand(cmd) then begin
          EConsole.Lines.Add('User: '+cmd);
        end;
      end;
      Key:=0;
      SendMessage(EConsole.Handle,WM_CHAR,ord('>'),0);
    end;
    VK_UP: Key:=0;
    VK_DOWN: Key:=0;
    192: Key:=0;
  end;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    192:                                    // ~
      ConsoleItemClick(Self);
  end;
end;

procedure TMainForm.EConsoleKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = '`') or (Key = '~') then
    Key:=#27;
end;


(*
  01.08.2001 - Deadline coming ...
*)

type
    PBatchModel = ^TBatchModel;
    TBatchModel = record
      ModelName : array[0..255] of char;
      Model     : PModel;
      B         : array[0..127] of double;
      Y         : double;
      Normalize : Boolean;
      OneSided  : Boolean;
      Min, Max  : double;
      C,R,W     : double;
    end;

var
   m_comb : TList;

function MyStrToFloat(s:string;var x:double):boolean;
begin
  try
    x := StrToFloat(s);
  except
    on EConvertError do
    begin
      { Convert '.' to ',' }
      while Pos('.', S) > 0 do
        S[Pos('.', S)] := ',';
        try
          x := StrToFloat(s);
        except
        on EConvertError do
        begin
          Result := False;
          Exit;
        end;
      end;
    end;
  end;
  Result := True;
end;

function LoadBatch(fname:PChar):Boolean;
label
  done_model;
label
  only_model;
var
  i,blen : integer;
  mdl : TBatchModel;
  p,ps : parser_p;
  s : string;
begin
  if m_comb <> nil then
  begin
    for i:=0 to m_comb.Count - 1 do
    begin
      if m_comb.Items[i] <> nil then
      begin
        FreeMem(m_comb.Items[i]);
        m_comb.Items[i] := nil;
      end;
    end;
    m_comb.Free;
  end;
  Result := False;
  if fname = nil then
    Exit;
  if not FileExists(fname) then
  begin
    ShowMessageFmt(' "%s"   !',[fname]);
    Exit;
  end;
  p:=tokStartFile(fname,' ');
  if p=nil then
  begin
    con_lprintf(LOG_ERROR,'LoadBatch(): empty file "%s"'+#13,[fname]);
    Exit;
  end;
  m_comb:=TList.Create;
  while tokString(p) <> nil do
  begin
    if StrLen(tokLast(p)) = 0 then
      continue;
    // Load next model
    ps := tokStart(tokLast(p),' ');
    ZeroMemory(@mdl,sizeof(TBatchModel));
    if StrComp(tokNext(ps),'model') <> 0 then
    begin
      con_lprintf(LOG_ERROR,'LoadBatch(): expected "model", found %s'+#13,[tokLast(ps)]);
      goto done_model;
    end;
    StrLCopy(mdl.ModelName,tokNext(ps),256);
    con_lprintf(LOG_INFO,'LoadBatch(): parsing model "%s"'+#13,[mdl.ModelName]);
    mdl.Model:=MainForm.Modules.Models[MainForm.Modules.ModelByName(mdl.ModelName)];
    if mdl.Model = nil then
    begin
      con_lprintf(LOG_ERROR,'LoadBatch(): model %s not found'+#13,[mdl.ModelName]);
      goto done_model;
    end;
    // Parse model's args
    blen:=mdl.Model^.AddenCount;
    con_lprintf(LOG_INFO,'LoadBatch(): model "%s" needs %d args; ',[mdl.ModelName,blen]);
    Result := True;
    mdl.Normalize:=True;
    for i:=1 to blen do
    begin
      s:=tokNext(ps);

(*
      try
        mdl.B[i-1] := StrToFloat(s);
      except
        on EConvertError do
        begin
          con_lprintf(LOG_ERROR,'couldn''t parse arg #%d'+#13,[i]);
          Result:=False;
        end;
      end;
*)
      Result := MyStrToFloat(s, mdl.B[i-1]);
      if not Result then
        goto done_model;
      con_lprintf(LOG_INFO,'[%d]=%g; ',[i,mdl.B[i-1]]);
    end;
    con_lputs(LOG_INFO,#13);
    // Parse combine info
    if(tokNext(ps) = nil) then
    begin
         goto only_model;
    end;
    if StrComp(tokLast(ps),'combine1') = 0 then
    begin
      // one - sided
      mdl.OneSided:=True;
      try
        mdl.Min := StrToFloat(tokNext(ps));
        mdl.Max := StrToFloat(tokNext(ps));
        mdl.C := 0;
        mdl.R := StrToFloat(tokNext(ps));
        mdl.W := StrToFloat(tokNext(ps));
      except
        on EConvertError do
        begin
          con_lputs(LOG_ERROR,'LoadBatch(): couldn''t parse combine info'+#13);
          Result:=False;
        end;
      end;
      if not Result then
        goto done_model;
    end
    else
    begin
      if StrComp(tokLast(ps),'combine2') = 0 then
      begin
        // two - sided
        mdl.OneSided:=False;
        try
          mdl.Min := StrToFloat(tokNext(ps));
          mdl.C := StrToFloat(tokNext(ps));
          mdl.Max := StrToFloat(tokNext(ps));
          mdl.R := StrToFloat(tokNext(ps));
          mdl.W := StrToFloat(tokNext(ps));
        except
          on EConvertError do
          begin
            con_lputs(LOG_ERROR,'LoadBatch(): couldn''t parse combine info'+#13);
            Result:=False;
          end;
        end;
        if not Result then
          goto done_model;
      end
      else
      begin
only_model:;
        // usual mean
        mdl.Normalize:=False;
        con_lprintf(LOG_INFO,'LoadBatch(): assuming the only model "%s", combination disabled.'+#13,[mdl.ModelName]);
        if m_comb.Count > 0 then
          con_lputs(LOG_WARNING,'LoadBatch(): warning: other models will be ignored !'+#13);
        // skip to eof
        i:=0;
        while tokString(p) <> nil do
          i:=i+1;
        con_lprintf(LOG_WARNING,'LoadBatch(): incomplete batch warning: skipped %d lines'+#13,[i]);
      end;
    end;
    // all ok, all to list
    m_comb.Add(AllocMem(sizeof(TBatchModel)));
    CopyMemory(m_comb.Items[m_comb.Count-1],@mdl,sizeof(TBatchModel));
done_model:
    tokFree(ps);
  end;
  tokFree(p);
  Result := m_comb.Count >= 1;
end;

{$OPTIMIZATION OFF}

type
  TMMInfo = record
    a1, a2, a3, Y : double;
  end;
var
  Min, Max : TMMInfo;

procedure clearmminfo;
begin
  Min.Y := 1e38;
  Max.Y := - 1e38;
end;

procedure checkmmpoint(a1,a2,a3,Y:double);
begin
  if Y < Min.Y then
  begin
    Min.Y:=Y;
    Min.a1:=a1;
    Min.a2:=a2;
    Min.a3:=a3;
  end;
  if Y > Max.Y then
  begin
    Max.Y:=Y;
    Max.a1:=a1;
    Max.a2:=a2;
    Max.a3:=a3;
  end;
end;

procedure batch_combine(var a1,a2,a3,res:double);stdcall;
var
  mdl : PBatchModel;
  i : integer;
  args      : array[0..2] of double;
  y,p,ws    : double;
begin
  args[0]:=a1;
  args[1]:=a2;
  args[2]:=a3;
  p := 1;
  ws := 0;
  for i:=0 to m_comb.Count-1 do
  begin
    mdl:=PBatchModel(m_comb.Items[i]);
    mdl^.Model^.Value(@args,@mdl^.B,y);
    if not mdl^.Normalize then
    begin
      res := y;
      checkmmpoint(a1,a2,a3,res);
      Exit;
    end;

    if mdl^.OneSided then
    begin
      if y <= mdl^.Min then
        y:=0 else
      if y >= mdl^.Max then
        y:=1 else
      y:=power((y-mdl^.Min)/(mdl^.Max-mdl^.Min),mdl^.R);
    end
    else
    begin
      if (y <= mdl^.Min) or (y >= mdl^.Max) then
        y:=0 else
      begin
        if y <= mdl^.C then
          y:=power((y-mdl^.Min)/(mdl^.C-mdl^.Min),mdl^.R)
        else
          y:=power((mdl^.Max-y)/(mdl^.Max-mdl^.C),mdl^.R);
      end;
    end;
    ws := ws + mdl^.W;
    p := p * power(y,mdl^.W);
  end;
  res:=power(p,1/ws);
  checkmmpoint(a1,a2,a3,res);
end;

{$OPTIMIZATION ON}

procedure TMainForm.BatchItemClick(Sender: TObject);
var
  od : TOpenDialog;
begin
  od:=TOpenDialog.Create(Self);
  od.InitialDir:=GetCurrentDir;
  if not od.Execute then
  begin
    od.Free;
    Exit;
  end;
  ExecuteBatch(od.FileName);
  od.Free;
end;

procedure TMainForm.ExecuteBatch(BatchFileName : string);
var
  center : vec3_t;
  R,a : vec_t;
begin
  if LoadBatch(PChar(BatchFileName)) then
  begin
    if profile <> nil then
      ppFree(profile);
    profile:=nil;
    if Bitmap.Width <> FormatDialog.SEViewWidth.Value then
      Bitmap.Width:=FormatDialog.SEViewWidth.Value;
    if Bitmap.Height <> FormatDialog.SEViewHeight.Value then
      Bitmap.Height:=FormatDialog.SEViewHeight.Value;
    if OutPanel.Width <> FormatDialog.SEViewWidth.Value then
      OutPanel.Width:=FormatDialog.SEViewWidth.Value;
    if OutPanel.Height <> FormatDialog.SEViewHeight.Value then
      OutPanel.Height:=FormatDialog.SEViewHeight.Value;
    R:=Bitmap.Width/2; ///(2*sqr(cos(Pi/6)));
    center[0]:=Bitmap.Width/2;
    center[1]:=Bitmap.Height/2+Bitmap.Height/10;
    center[2]:=0;
    ChooseMdlUnit.progress:=0;
    a:=-Pi/2;
    clearmminfo;
    if ChooseMdlParamsDialog.CBScalesOnly.Checked then
      profile:=ppCreateTriangular(center,R,a,nil,@batch_combine,ChooseMdlParamsDialog.lines,0)
    else
      profile:=ppCreateTriangular(center,R,a,@ChooseMdlParamsDialog.abc,@batch_combine,ChooseMdlParamsDialog.lines,0);
    StatusBar.SimpleText:='';
    if profile = nil then begin
      ShowMessage(' .   .');
      PatBlt(Bitmap.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,WHITENESS);
      BitBlt(OutPanel.Canvas.Handle,0,0,Bitmap.Width,Bitmap.Height,Bitmap.Canvas.Handle,0,0,SRCCOPY);
    end
    else
    begin
      DrawPPOnCanvas(profile,Bitmap.Canvas,Bitmap.Width,Bitmap.Height,ChooseMdlParamsDialog.split);
      OutPanelPaint(Self);
      con_printf('Minimal %3.5g at [%3.5g;%3.5g;%3.5g]'+#13,[Min.Y,Min.a1,Min.a2,Min.a3]);
      con_printf('Maximal %3.5g at [%3.5g;%3.5g;%3.5g]'+#13,[Max.Y,Max.a1,Max.a2,Max.a3]);
    end;
  end;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  if ParamCount = 1 then
    ExecuteBatch(ParamStr(1));
end;

end.

