unit v_lib;

{
<HTML>
<PRE>
}

interface

uses
    Math, SysUtils;

{$DEFINE MSC}

const
    V_LIB_DLL = 'v_lib.dll';

{   ===================================================================== }
{   u_typedef.hpp                                                         }
const
    BOGUS_LIMIT                         = 1024*1024-1;

    MAXSHORT                            = $7FFF;
    MINSHORT                            = $8000;
    MAXLONG                             = $7FFFFFFF;
    MINLONG                             = $80000000;
    MAXFLOAT                            = 3.4e+38;
    MAXDOUBLE                           = 1.7e+308;
    FLOAT_ZERO_EPSILON                  = 1e-6;
    FLOAT_ZERO_EPSILON_SQUARE           = 1e-12;

type
    u_8                                 = Byte;
    i_8                                 = ShortInt;
    u_16                                = Word;
    i_16                                = SmallInt;
    u_32                                = LongInt;
    i_32                                = Integer;
    i_64                                = Comp;
    u_64                                = Comp;
    pvoid                               = Pointer;
    pbyte                               = ^u_8;
    pword                               = ^u_16;
    pdword                              = ^u_32;

    i_32_array_t                        = array[0..BOGUS_LIMIT] of i_32;
    i_32_array_p                        = ^i_32_array_t;
    i_32_array_p_array_t                = array[0..BOGUS_LIMIT]
                                          of i_32_array_p;
    i_32_array_p_array_p                = ^i_32_array_p_array_t;
    u_8_array_t                         = array[0..BOGUS_LIMIT] of u_8;
    u_8_array_p                         = ^u_8_array_t;

{$IFDEF DOUBLEVEC}
    scalar_t                            = double;
{$ELSE}
    scalar_t                            = single;
{$ENDIF}
    scalar_p                            = ^scalar_t;
    vec_t                               = scalar_t;
    vec_p                               = ^vec_t;
    vec2_t                              = array[0..1] of vec_t;
    vec2_p                              = ^vec2_t;
    vec3_t                              = array[0..2] of vec_t;
    vec3_p                              = ^vec3_t;
    vec4_t                              = array[0..3] of vec_t;
    vec4_p                              = ^vec4_t;
    rgb_t                               = vec_t;
    rgba_t                              = vec4_t;
    bytergb_t                           = array[0..2] of byte;
    bytergb_p                           = ^bytergb_t;
    bytergb_t_array_t                   = array[0..BOGUS_LIMIT]
                                          of bytergb_t;
    bytergb_t_array_p                   = ^bytergb_t_array_t;
    bytergba_t                          = array[0..3] of byte;
    bytergba_p                          = ^bytergba_t;
    bytergba_t_array_t                  = array[0..BOGUS_LIMIT]
                                          of bytergba_t;
    bytergba_t_array_p                  = ^bytergba_t_array_t;
    matrix2_t                           = array[0..1,0..1] of vec_t;
    matrix2_p                           = ^matrix2_t;
    matrix3_t                           = array[0..2,0..2] of vec_t;
    matrix3_p                           = ^matrix3_t;
    matrix4_t                           = array[0..3,0..3] of vec_t;
    matrix4_p                           = ^matrix4_t;
    matrix3x4_t                         = array[0..2,0..3] of vec_t;
    matrix3x4_p                         = ^matrix3x4_t;
    transform_t                         = matrix3x4_t;
    transform_p                         = ^transform_t;
    vec_t_array_t                       = array[0..BOGUS_LIMIT] of vec_t;
    vec_t_array_p                       = ^vec_t_array_t;
    vec_t_array_p_array_t               = array[0..BOGUS_LIMIT]
                                          of vec_t_array_p;
    vec_t_array_p_array_p               = ^vec_t_array_p_array_t;
    vec2_t_array_t                      = array[0..BOGUS_LIMIT] of vec2_t;
    vec2_t_array_p                      = ^vec2_t_array_t;
    vec3_t_array_t                      = array[0..BOGUS_LIMIT] of vec3_t;
    vec3_t_array_p                      = ^vec3_t_array_t;
    vec4_t_array_t                      = array[0..BOGUS_LIMIT] of vec4_t;
    vec4_t_array_p                      = ^vec4_t_array_t;

{   ===================================================================== }
{   u_x86.hpp                                                             }
//  For copmatibility only: NO speed boost
function  u_abs(x:vec_t):vec_t;
procedure u_fsincos(var s,c:vec_t;r:vec_t);
{$IFDEF MSC}
procedure u_fill(dst:pvoid;c:u_32;n:i_32);stdcall;external V_LIB_DLL;
procedure u_filldw(dst:pvoid;c:u_32;dwords:i_32);stdcall;external V_LIB_DLL;
procedure u_zero(dst:pvoid;n:i_32);stdcall;external V_LIB_DLL;
procedure u_zerodw(dst:pvoid;dwords:i_32);stdcall;external V_LIB_DLL;
procedure u_copy(dst,src:pvoid;n:i_32);stdcall;external V_LIB_DLL;
procedure u_copydw(dst,src:pvoid;dwords:i_32);stdcall;external V_LIB_DLL;
{$ENDIF}

{   ===================================================================== }
{   u_math.hpp                                                            }
const
    M_PI                                = 3.14159265358979323846;
    M_2PI                               = 6.28318530717958647692;
    M_PI2                               = 1.5707963267949;
    M_PI3                               = 1.0471975511966;
    M_PI4                               = 0.7853981633975;
    M_PI6                               = 0.5235987755983;
    //    ;     -   
    ANGLE_0                             = 0.0;
    ANGLE_1                             = 0.017453292;
    ANGLE_5                             = 0.087266462;
    ANGLE_10                            = 0.174532925;
    ANGLE_15                            = 0.261799387;
    ANGLE_20                            = 0.392699081;
    ANGLE_30                            = 0.523598775;
    ANGLE_45                            = 0.785398163;
    ANGLE_60                            = 1.047197551;
    ANGLE_72                            = 1.256637061;
    ANGLE_90                            = 1.570796327;
    ANGLE_120                           = 2.094395102;
    ANGLE_135                           = 2.35619449;
    ANGLE_144                           = 2.513274123;
    ANGLE_180                           = 3.141592653;
    ANGLE_225                           = 3.926990817;
    ANGLE_270                           = 4.71238898;
    ANGLE_315                           = 5.497787144;
    ANGLE_360                           = 6.283185307;
    RAD_IN_ANGLE                        = 0.01745329251994;
    ANGLE_TO_RAD                        = 0.01745329251994;
    ANGLE_IN_RAD                        = 57.29577951618;
    RAD_TO_ANGLE                        = 57.29577951618;
    PITCH                               = 0;
    YAW                                 = 1;
    ROLL                                = 2;

function  ToRadians(a:vec_t):vec_t;
function  ToDegrees(a:vec_t):vec_t;
procedure vec2Set(var vec:vec2_t;x,y:vec_t);
procedure vec3Set(var vec:vec3_t;x,y,z:vec_t);
procedure vec4Set(var vec:vec4_t;x,y,z,w:vec_t);
procedure vec2Zero(var vec:vec2_t);
procedure vec3Zero(var vec:vec3_t);
procedure vec4Zero(var vec:vec4_t);
procedure vec2Copy(var dst,src:vec2_t);
procedure vec3Copy(var dst,src:vec3_t);
procedure vec4Copy(var dst,src:vec4_t);
procedure vec2Add(var dst,v1,v2:vec2_t);
procedure vec3Add(var dst,v1,v2:vec3_t);
procedure vec4Add(var dst,v1,v2:vec4_t);
procedure vec2Sub(var dst,v1,v2:vec2_t);
procedure vec3Sub(var dst,v1,v2:vec3_t);
procedure vec4Sub(var dst,v1,v2:vec4_t);
procedure vec2Scale(var dst,src:vec2_t;scale:vec_t);
procedure vec3Scale(var dst,src:vec3_t;scale:vec_t);
procedure vec4Scale(var dst,src:vec4_t;scale:vec_t);
procedure vec2Negate(var dst,src:vec2_t);
procedure vec3Negate(var dst,src:vec3_t);
procedure vec4Negate(var dst,src:vec4_t);
procedure vec2NegateSelf(var v:vec2_t);
procedure vec3NegateSelf(var v:vec3_t);
procedure vec4NegateSelf(var v:vec4_t);
function  vec2Dot(var v1,v2:vec2_t):vec_t;
function  vec3Dot(var v1,v2:vec3_t):vec_t;
function  vec4Dot(var v1,v2:vec4_t):vec_t;
function  vec2Abs(var vec:vec2_t):vec_t;
function  vec3Abs(var vec:vec3_t):vec_t;
function  vec4Abs(var vec:vec4_t):vec_t;
procedure vec2Unit(var dst,vec:vec2_t);
procedure vec3Unit(var dst,vec:vec3_t);
procedure vec4Unit(var dst,vec:vec4_t);
procedure vec4Homogenize(var dst,vec:vec4_t);
function  vec2Cross(var v1,v2:vec2_t):vec_t;
function  vec3Mix(var v1,v2,v3:vec3_t):vec_t;
function  vec2MinInd(var vec:vec2_t):integer;
function  vec2MaxInd(var vec:vec2_t):integer;
function  vec3MinInd(var vec:vec3_t):integer;
function  vec3MaxInd(var vec:vec3_t):integer;
function  vec4MinInd(var vec:vec4_t):integer;
function  vec4MaxInd(var vec:vec4_t):integer;
function  IsZero(v:vec_t):Boolean;
function  vec2IsZero(var vec:vec2_t):Boolean;
function  vec3IsZero(var vec:vec3_t):Boolean;
function  vec4IsZero(var vec:vec4_t):Boolean;
function  vec2IsEqual(var v1,v2:vec2_t):Boolean;
function  vec3IsEqual(var v1,v2:vec3_t):Boolean;
function  vec4IsEqual(var v1,v2:vec4_t):Boolean;
function  vec2IsEqualEpsilon(var a,b:vec2_t;epsilon:vec_t):Boolean;
function  vec3IsEqualEpsilon(var a,b:vec3_t;epsilon:vec_t):Boolean;
function  vec4IsEqualEpsilon(var a,b:vec4_t;epsilon:vec_t):Boolean;
procedure vec2ScaleByVec2(var dst,v1,v2:vec2_t);
procedure vec3ScaleByVec3(var dst,v1,v2:vec3_t);
procedure vec4ScaleByVec4(var dst,v1,v2:vec4_t);
procedure vec2ZoomIF(var zoom,initial,final:vec2_t);
procedure vec3ZoomIF(var zoom,initial,final:vec3_t);
procedure vec4ZoomIF(var zoom,initial,final:vec4_t);
procedure vec2ToRadians(var dst,src:vec2_t);
procedure vec3ToRadians(var dst,src:vec3_t);
procedure vec4ToRadians(var dst,src:vec4_t);
procedure vec2ToDegrees(var dst,src:vec2_t);
procedure vec3ToDegrees(var dst,src:vec3_t);
procedure vec4ToDegrees(var dst,src:vec4_t);
procedure mat2Zero(var mat:matrix2_t);
procedure mat3Zero(var mat:matrix3_t);
procedure mat4Zero(var mat:matrix4_t);
procedure mat3x4Zero(var mat:matrix3x4_t);
procedure mat2Copy(var dst,src:matrix2_t);
procedure mat3Copy(var dst,src:matrix3_t);
procedure mat4Copy(var dst,src:matrix4_t);
procedure mat3x4Copy(var dst,src:matrix3x4_t);
procedure mat2Unit(var mat:matrix2_t);
procedure mat3Unit(var mat:matrix3_t);
procedure mat4Unit(var mat:matrix4_t);
procedure mat3x4Unit(var mat:matrix3x4_t);
procedure mat2Transpose(var dst,src:matrix2_t);
procedure mat2TransposeSelf(var mat:matrix2_t);
procedure vec3Transform(var vec:vec3_t;var mat:matrix3x4_t);
//        dst := transform x dst;
procedure mat3x4ConcatBefore(var dst,transform:matrix3x4_t);
//        dst := dst x transform;
procedure mat3x4ConcatAfter(var dst,transform:matrix3x4_t);
procedure rgbScalarFromByte(var rgb:vec3_t;var brgb:bytergb_t);
procedure rgbaScalarFromByte(var rgba:vec4_t;var brgba:bytergba_t);
procedure rgbByteFromScalar(var brgb:bytergb_t;var rgb:vec3_t);
procedure rgbaByteFromScalar(var brgba:bytergba_t;var rgba:vec4_t);
procedure vec3Cross(var dst,v1,v2:vec3_t);stdcall;external V_LIB_DLL;
procedure mat4FromMat3(var mat4:matrix4_t;var mat3:matrix3_t);stdcall;
  external V_LIB_DLL;
procedure mat3FromMat4(var mat3:matrix3_t;var mat4:matrix4_t);stdcall;
  external V_LIB_DLL;
procedure mat3Transpose(var dst,src:matrix3_t);stdcall;external V_LIB_DLL;
procedure mat4Transpose(var dst,src:matrix4_t);stdcall;external V_LIB_DLL;
procedure mat4TransposeSelf(var mat:matrix4_t);stdcall;external V_LIB_DLL;
procedure mat3TransposeSelf(var mat:matrix3_t);stdcall;external V_LIB_DLL;
procedure mat2ByVec2(var dst:vec2_t;var mat:matrix2_t;var v:vec2_t);stdcall;
  external V_LIB_DLL;
procedure mat3ByVec3(var dst:vec3_t;var mat:matrix3_t;var v:vec3_t);stdcall;
  external V_LIB_DLL;
procedure mat3x4ByVec3(var dst:vec3_t;var mat:matrix3x4_t;var v:vec3_t);
  stdcall;external V_LIB_DLL;
procedure mat4ByVec4(var dst:vec4_t;var mat:matrix4_t;var v:vec4_t);
  stdcall;external V_LIB_DLL;
procedure mat2ByMat2(var dst,m1,m2:matrix2_t);stdcall;external V_LIB_DLL;
procedure mat3ByMat3(var dst,m1,m2:matrix3_t);stdcall;external V_LIB_DLL;
procedure mat4ByMat4(var dst,m1,m2:matrix4_t);stdcall;external V_LIB_DLL;
procedure mat3x4ByMat3x4(var dst,m1,m2:matrix3x4_t);stdcall;
  external V_LIB_DLL;
procedure vecs3Transform(vecs:vec3_t_array_p;var mat:matrix3x4_t;
  n:integer);stdcall;external V_LIB_DLL;
procedure mat2Solve(var a:matrix2_t;var b:vec2_t);stdcall;
  external V_LIB_DLL;
procedure mat2SolveVV(var r1,r2,b:vec2_t);stdcall;external V_LIB_DLL;
procedure mat3Solve(var a:matrix3_t;var b:vec3_t);stdcall;
  external V_LIB_DLL;
procedure mat4Solve(var a:matrix4_t;var b:vec4_t);stdcall;
  external V_LIB_DLL;

{   ===================================================================== }
{   u_vector.hpp                                                          }
type
  vector_p                              = ^vector_t;
  vector_t                              = record
    size                                : integer;
    elements                            : vec_t_array_p;
  end;

function  vecAlloc(size:integer):vector_p;
procedure vecCopy(dst,src:vector_p);
procedure vecFree(v:vector_p);
function  vecDuplicate(v:vector_p;keepon:Boolean):vector_p;
procedure vecZero(v:vector_p);
function  vecDot(v1,v2:vector_p):vec_t;
function  vecAbs(v:vector_p):vec_t;
procedure vecScale(dest,v:vector_p;x:vec_t);
function  vecUnit(v:vector_p):Boolean;
procedure vecAdd(dest,v1,v2:vector_p);stdcall;external V_LIB_DLL;
procedure vecSub(dest,v1,v2:vector_p);stdcall;external V_LIB_DLL;
procedure vecNegate(v:vector_p);stdcall;external V_LIB_DLL;
function  vecCompare(v1,v2:vector_p):Boolean;stdcall;external V_LIB_DLL;
function  vecMin(v:vector_p):integer;stdcall;external V_LIB_DLL;
function  vecMax(v:vector_p):integer;stdcall;external V_LIB_DLL;

{   ===================================================================== }
{   u_matrix.hpp                                                          }
type
  matrix_p = ^matrix_t;
  matrix_t = record
    rows,cols                           : integer;
    data                                : vec_t_array_p;
    cell                                : vec_t_array_p_array_p;
  end;

function  matAlloc(rows,cols:integer):matrix_p;stdcall;external V_LIB_DLL;
function  matDuplicate(m:matrix_p;keepon:Boolean):matrix_p;stdcall;
  external V_LIB_DLL;
function  matExtractRow(dest:vector_p;m:matrix_p;row:integer):vector_p;
  stdcall;external V_LIB_DLL;
function  matExtractCol(dest:vector_p;m:matrix_p;row:integer):vector_p;
  stdcall;external V_LIB_DLL;
procedure matUnit(m:matrix_p);stdcall;external V_LIB_DLL;
procedure matAdd(dest,m1,m2:matrix_p);stdcall;external V_LIB_DLL;
procedure matSub(dest,m1,m2:matrix_p);stdcall;external V_LIB_DLL;
procedure matNegate(dest,m:matrix_p);stdcall;external V_LIB_DLL;
function  matCompare(m1,m2:matrix_p):Boolean;stdcall;external V_LIB_DLL;
procedure matMin(m:matrix_p;var row,col:integer);stdcall;external V_LIB_DLL;
procedure matMax(m:matrix_p;var row,col:integer);stdcall;external V_LIB_DLL;
function  matByVec(dest:vector_p;m:matrix_p;v:vector_p):vector_p;stdcall;
  external V_LIB_DLL;
function  matByMat(dest,ml,mr:matrix_p):matrix_p;stdcall;external V_LIB_DLL;
procedure matTranspose(m:matrix_p);stdcall;external V_LIB_DLL;
function  matCreateTransposed(m:matrix_p;keepon:Boolean):matrix_p;stdcall;
  external V_LIB_DLL;
function  matSolve(a:vector_p;b:vector_p;keepon:Boolean):vector_p;stdcall;
  external V_LIB_DLL;
function  matInverse(m:matrix_p;keepon:Boolean):matrix_p;stdcall;
  external V_LIB_DLL;
function  matDeterm(m:matrix_p;keepon:Boolean):vec_t;

{   ===================================================================== }
{   u_plane.hpp                                                           }
type
  line2_p                               = ^line2_t;
  line2_t                               = record
    case integer of
    1:
      (
        normal                          : vec2_t;
        dist                            : vec_t;
      );
    2:
      (
        abc                             : vec3_t;
      );
  end;

  line2_t_array_t                       = array[0..BOGUS_LIMIT] of line2_t;
  line2_t_array_p                       = ^line2_t_array_t;

  ray2_p                                = ^ray2_t;
  ray2_t                                = record
    origin                              : vec2_t;
    line                                : line2_t;
  end;
  ray2_t_array_t                        = array[0..BOGUS_LIMIT] of ray2_t;
  ray2_t_array_p                        = ^ray2_t_array_t;

  poly2_p                               = ^poly2_t;
  poly2_t                               = record
    numpoints                           : integer;
    count                               : integer;
    size                                : integer;
    rays                                : ray2_t_array_p;
  end;

procedure line2Cross(var point:vec2_t;l1,l2:line2_p);stdcall;
  external V_LIB_DLL;
procedure line2FromPoints(line:line2_p;var p1,p2:vec2_t);stdcall;
  external V_LIB_DLL;
function  poly2Alloc(points:integer):poly2_p;stdcall;external V_LIB_DLL;
function  poly2Duplicate(ref:poly2_p):poly2_p;stdcall;external V_LIB_DLL;
procedure poly2ComputeLines(w:poly2_p);stdcall;external V_LIB_DLL;
function  poly2PointInside(w:poly2_p;var p:vec2_t):Boolean;stdcall;
  external V_LIB_DLL;

{   ===================================================================== }
{   u_space.hpp                                                           }

type
  plane3_p                              = ^plane3_t;
  plane3_t                              = record
    case integer of
    1:
      (
        normal                          : vec3_t;
        dist                            : vec_t;
      );
    2:
      (
        abcd                            : vec4_t;
      );
  end;

  line3_p                               = ^line3_t;
  line3_t                               = record
    origin                              : vec3_t;
    dir                                 : vec3_t;
  end;

  vecdir_t                              = integer;

const
  dir_x = 1;
  dir_y = 2;
  dir_z = 3;
  dir_anyx = 4;
  dir_anyy = 5;
  dir_anyz = 6;

procedure mat3RotateX(var mat:matrix3_t;radians:vec_t);stdcall;
  external V_LIB_DLL;
procedure mat3RotateY(var mat:matrix3_t;radians:vec_t);stdcall;
  external V_LIB_DLL;
procedure mat3RotateZ(var mat:matrix3_t;radians:vec_t);stdcall;
  external V_LIB_DLL;
procedure mat3x4RotateX(var mat:matrix3x4_t;radians:vec_t);stdcall;
  external V_LIB_DLL;
procedure mat3x4RotateY(var mat:matrix3x4_t;radians:vec_t);stdcall;
  external V_LIB_DLL;
procedure mat3x4RotateZ(var mat:matrix3x4_t;radians:vec_t);stdcall;
  external V_LIB_DLL;
procedure mat3Rotate(var mat:matrix3_t;var dir:vec3_t;radians:vec_t);
  stdcall;external V_LIB_DLL;
procedure mat3x4Rotate(var mat:matrix3x4_t;var dir:vec3_t;
  radians:vec_t);stdcall;external V_LIB_DLL;
procedure vec3RotateAround(var dst,dir,point:vec3_t;radians:vec_t);
  stdcall;external V_LIB_DLL;
procedure vec3RotateAroundQ(var dst,dir,point:vec3_t;radians:vec_t);
  stdcall;external V_LIB_DLL;
procedure mat3Scale(var m:matrix3_t;var scale:vec3_t);stdcall;
  external V_LIB_DLL;
procedure mat3FromDirAndUp(var mat:matrix3_t;var dir,up:vec3_t);stdcall;
  external V_LIB_DLL;
procedure mat3x4Scale(var mL:matrix3x4_t;var scale:vec3_t);stdcall;
  external V_LIB_DLL;
procedure mat3x4Translate(var m:matrix3x4_t;var t:vec3_t);stdcall;
  external V_LIB_DLL;
procedure mat3x4FromDirAndUp(var mat:matrix3x4_t;var dir,up:vec3_t);
  stdcall;external V_LIB_DLL;
procedure mat3x4FromPointDirAndUp(var mat:matrix3x4_t;
  var pt,dir,up:vec3_t);stdcall;external V_LIB_DLL;
procedure line3FromPoints(line:line3_p;var p1,p2:vec3_t);stdcall;
  external V_LIB_DLL;
procedure line3FromDirAndOrigin(line:line3_p;var dir,origin:vec3_t);
  stdcall;external V_LIB_DLL;
procedure vec3FromLineAndPlane(var pt:vec3_t;line:line3_p;plane:plane3_p);
  stdcall;external V_LIB_DLL;
procedure vec3PlaneReflection(var dst:vec3_t;pl:plane3_p;var src:vec3_t);
  stdcall;external V_LIB_DLL;
procedure plane3FromPoints(dest:plane3_p;var a,b,c:vec3_t);stdcall;
  external V_LIB_DLL;
procedure plane3FromLines(dest:plane3_p;l1,l2:line3_p);stdcall;
  external V_LIB_DLL;
procedure projectPoint3OnPlane3(var proj:vec3_t;var pt:vec3_t;
  plane:plane3_p);stdcall;external V_LIB_DLL;
procedure projectPoint3OnPlane3Q(var proj:vec3_t;var pt:vec3_t;
  plane:plane3_p);stdcall;external V_LIB_DLL;
procedure vec3Perpendicular(var dest,v:vec3_t);stdcall;external V_LIB_DLL;
procedure vec3PerpendicularQ(var dest,v:vec3_t);stdcall;external V_LIB_DLL;
function  distFromPlane3(var pt:vec3_t;pl:plane3_p):vec_t;

{   ===================================================================== }
{   u_dmodes.hpp                                                          }

const
  MAX_DISPLAY_MODES                     = 128;
  MAX_REFRESH                           = 64;
  EDM_BPP_256COLOR                      = 1;
  EDM_BPP_HICOLOR                       = 2;
  EDM_BPP_TRUECOLOR                     = 4;
  EDM_BPP_ALL                           = 7;

  type
    modedesc_p                          = ^modedesc_t;
    modedesc_t                          = record
      xres, yres,bpp                    : integer;
      refcount                          : integer;
      refresh                           : array[0..MAX_REFRESH-1]
                                          of integer;
    end;

    displaymodes_p                      = ^displaymodes_t;
    displaymodes_t                      = record
      count                             : integer;
      modes                             : array[0..MAX_DISPLAY_MODES-1]
                                          of modedesc_t;
    end;

procedure dispEnumModes(modes:displaymodes_p;enumflags:integer);stdcall;
  external V_LIB_DLL;
procedure dispGetMode(var x,y,bpp:integer);stdcall;external V_LIB_DLL;
function  dispSetMode(x,y,bpp,refresh:integer):Boolean;stdcall;
  external V_LIB_DLL;
procedure dispRestoreMode;stdcall;external V_LIB_DLL;

{   ===================================================================== }
{   u_time.hpp                                                            }
procedure timeStart;stdcall;external V_LIB_DLL;
procedure timeStop;stdcall;external V_LIB_DLL;
function  timeMilliseconds:integer;stdcall;external V_LIB_DLL;
procedure timeDelay(Milliseconds:integer);stdcall;external V_LIB_DLL;

{   ===================================================================== }
{   u_image.hpp                                                           }
const
  rgbared                               = 0;
  rgbagreen                             = 1;
  rgbablue                              = 2;
  rgbaalpha                             = 3;

type
  rgbachannel_t                         = integer;

  miphead_p                             = ^miphead_t;
  miphead_t                             = record
    width,height,levels                 : integer;
    level                               : u_8_array_p;
  end;

function  targaLoad(name:pchar;var width,height:integer):pointer;stdcall;
  external V_LIB_DLL;
procedure targaSave(name:pchar;width,height:integer;pixels:pointer;
  include_alpha:Boolean);stdcall;external V_LIB_DLL;
function  bitmapLoad(name:pchar;var width,height:integer):pointer;stdcall;
  external V_LIB_DLL;
procedure bitmapSave(name:pchar;width,height:integer;pixels:pointer);
  stdcall;external V_LIB_DLL;
function  ppmLoad(name:pchar;var width,height:integer):pointer;stdcall;
  external V_LIB_DLL;
procedure ppmSave(name:pchar;width,height:integer;pixels:pointer);stdcall;
  external V_LIB_DLL;
function  rgbaLoad(name:pchar;var width,height:integer):pointer;stdcall;
  external V_LIB_DLL;
procedure rgbaSave(name:pchar;width,height:integer;pixels:pointer);stdcall;
  external V_LIB_DLL;
function  rgbaLoadChannel(name:pchar;var width,height:integer):pointer;
  stdcall;external V_LIB_DLL;
procedure rgbaSaveChannel(name:pchar;width,height:integer;ch_data:pointer);
  stdcall;external V_LIB_DLL;
procedure targaFree(pixels:pointer);stdcall;external V_LIB_DLL;
procedure bitmapFree(pixels:pointer);stdcall;external V_LIB_DLL;
procedure ppmFree(pixels:pointer);stdcall;external V_LIB_DLL;
procedure rgbaFree(pixels:pointer);stdcall;external V_LIB_DLL;
procedure rgbaSplitChannels(width,height:integer;pixels,r,g,b,a:pointer);
  stdcall;external V_LIB_DLL;
procedure rgbaCombineChannels(width,height:integer;pixels,r,g,b,a:pointer;
  sr,sg,sb,sa:vec_t);stdcall;external V_LIB_DLL;
function  mipAlloc(width,height,levels:integer):miphead_p;stdcall;
  external V_LIB_DLL;
function  mipLoadLevel(mip:miphead_p;rgbafile:pchar;level:integer):Boolean;
  stdcall;external V_LIB_DLL;
procedure mipSaveLevel(mip:miphead_p;rgbafile:pchar;level:integer);stdcall;
  external V_LIB_DLL;
function  mipBuild(rgbafile:pchar;levels:integer):miphead_p;stdcall;
  external V_LIB_DLL;
procedure mipSave(mipfile:pchar;mip:miphead_p);stdcall;external V_LIB_DLL;
function  mipLoad(mipfile:pchar):miphead_p;stdcall;external V_LIB_DLL;
procedure mipFree(mip:miphead_p);stdcall;external V_LIB_DLL;

{   ===================================================================== }
{   u_parse.hpp                                                           }

{ this is hack ... }

type
  parser_p                              = pointer;

function  tokStart(buffer,delim:pchar):parser_p;stdcall;external 'libv3.dll';
function  tokStartFile(fname,delim:pchar):parser_p;stdcall;external 'libv3.dll';
procedure tokFree(tok:parser_p);stdcall;external 'libv3.dll' name 'libvFree';
procedure tokRestart(tok:parser_p);stdcall;external 'libv3.dll';
function  tokPush(tok:parser_p):integer;stdcall;external 'libv3.dll';
function  tokPop(tok:parser_p):integer;stdcall;external 'libv3.dll';
procedure tokDelim(tok:parser_p;delim:pchar);stdcall;external 'libv3.dll';
function  tokNext(tok:parser_p):pchar;stdcall;external 'libv3.dll';
function  tokString(tok:parser_p):pchar;stdcall;external 'libv3.dll';
function  tokLast(tok:parser_p):pchar;stdcall;external 'libv3.dll';

{   ===================================================================== }
{   u_paklib.hpp                                                          }

const
  SEEK_SET                              = 0; // from stdio.h
  SEEK_CUR                              = 1;
  SEEK_END                              = 2;

function  fsInit(path:pchar):Boolean;stdcall;external V_LIB_DLL;
procedure fsShutdown;stdcall;external V_LIB_DLL;
function  fsPakfiles:integer;stdcall;external V_LIB_DLL;
function  fsNumRes(paknum:integer):integer;stdcall;external V_LIB_DLL;
function  fsResNum(paknum:integer;resname:pchar):integer;stdcall;
  external V_LIB_DLL;
function  fsResName(paknum,resnum:integer):pchar;stdcall;external V_LIB_DLL;
function  rload(unix_path_name:pchar;var sz:u_32):pointer;stdcall;
  external V_LIB_DLL;
procedure rfree(res:pointer);stdcall;external V_LIB_DLL;
function  rfind(unix_path_name:pchar):i_32;stdcall;external V_LIB_DLL;
procedure rclose(handle:i_32);stdcall;external V_LIB_DLL;
function  rsize(handle:i_32):integer;stdcall;external V_LIB_DLL;
function  rtell(handle:I_32):integer;stdcall;external V_LIB_DLL;
function  reof(handle:i_32):Boolean;stdcall;external V_LIB_DLL;
function  rseek(handle:i_32;offset:u_32;whence:integer):integer;stdcall;
  external V_LIB_DLL;
function  rread(handle:i_32;buffer:pointer;length:integer):integer;stdcall;
  external V_LIB_DLL;
function  rread8(handle:i_32):u_32;stdcall;external V_LIB_DLL;
function  rread16(handle:i_32):u_16;stdcall;external V_LIB_DLL;
function  rread32(handle:i_32):u_32;stdcall;external V_LIB_DLL;

{   ===================================================================== }
{   u_pgrid.hpp                                                           }

type
  itriple_t                             = array[0..2] of integer;
  itriple_t_array_t                     = array[0..BOGUS_LIMIT]
                                          of itriple_t;
  itriple_t_array_p                     = ^itriple_t_array_t;
  tripatch_p                            = ^tripatch_t;
  tripatch_t                            = record
    numvert                             : integer;
    numtri                              : integer;
    extinfo                             : integer;
    name                                : array[0..15] of char;
    base                                : array[0..2] of vec3_t;
    basenormal                          : vec3_t;
    vlist                               : vec3_t_array_p;
    tlist                               : itriple_t_array_p;
    clist                               : bytergba_t_array_p;
    nlist                               : vec3_t_array_p;
    fnlist                              : vec3_t_array_p;
    uvlist                              : vec2_t_array_p;
  end;

  patch_settings_p                      = ^patch_settings_t;
  patch_settings_t                      = record
    now_this_is_stub_only               : Boolean;
  end;

function  patAlloc(numvert,numtri:integer;name:pchar;extinfo:integer):
  tripatch_p;stdcall;external V_LIB_DLL;
function  patDuplicate(ref:tripatch_p):tripatch_p;stdcall;
  external V_LIB_DLL;
procedure patFree(patch:tripatch_p);stdcall;external V_LIB_DLL;
procedure patDefaultGeometry(patch:tripatch_p;settings:patch_settings_p);
  stdcall;external V_LIB_DLL;
function  patCreateRectangular(l,t,r,b:vec_t;hsplit,vsplit:integer;
  extinfo:Boolean):tripatch_p;stdcall;
  external V_LIB_DLL;
function  patCreateTriangular(var a,b,c:vec3_t;split:integer;
  extinfo:Boolean):tripatch_p;stdcall;external V_LIB_DLL;

{   ===================================================================== }
{   u_pprof.hpp                                                           }

const
    BASE_XOY                            = 0;
    BASE_XOZ                            = 1;
    BASE_YOZ                            = 2;

    MAX_GRAPH_LABEL                     = 64;
    LF_NOLABEL                          = 1;
    LF_DIRECTION                        = 2;
    LF_CENTER                           = 4;

type
    graph_label_p                       = ^graph_label_t;
    graph_label_t                       = record
      dir, center                       : vec3_t;
      flags                             : integer;
      elem_size                         : vec_t;
      value                             : vec_t;
      text                              : array[0..MAX_GRAPH_LABEL-1]
                                          of char;
    end;
    graph_label_t_array_t               = array[0..1024*1024-1]
                                          of graph_label_t;
    graph_label_t_array_p               = ^graph_label_t_array_t;

    wirechains_p                        = ^wirechains_t;
    wirechains_t                        = record
      numvert_total                     : integer;
      numchains                         : integer;
      vlist                             : vec3_t_array_p;
      numvert                           : i_32_array_p;
      clist                             : i_32_array_p_array_p;
      ilist                             : i_32_array_p;
      llist                             : graph_label_t_array_p;
    end;

    wirechains_p_array_t                = array[0..1024*1024-1]
                                          of wirechains_p;
    wirechains_p_array_p                = ^wirechains_p_array_t;

    planar_func_t            = procedure(var arg1,arg2,res:double);stdcall;
    spatial_func_t           = procedure(var arg1,arg2,res:double);stdcall;
    planar_func_p                       = ^planar_func_p;
    spatial_func_p                      = spatial_func_t;

    patchprofile_p = ^patchprofile_t;
    patchprofile_t = record
      planar       : planar_func_t;    // nil    
      spatial      : spatial_func_t;   // nil   
      baseplane    : integer;          // 0 - xOy, 1 - xOz, 2 - yOz
      base         : array[0..3] of vec3_t;
      fmin, fmax   : vec_t;            //   
      splits       : integer;          //   
      wires        : wirechains_p_array_p;  //  .
      chainvals    : vec_t_array_p;    //    
    end;

function  ppGridFunction(var l,t,r,b:vec_t;hsplit,vsplit:integer;
  settings:patch_settings_p;planar:planar_func_t;
  basenum:integer):tripatch_p;stdcall;external V_LIB_DLL
  name 'ppGridFunctionTHK';
function  ppGridSheffePolynom(var center:vec3_t;var radius,angle0:vec_t;
  split:integer;settings:patch_settings_p;gibbs_abc:vec3_t_array_p;
  sheffe:spatial_func_t;basenum:integer):tripatch_p;stdcall;
  external V_LIB_DLL name 'ppGridSheffePolynomTHK';
function  ppCreateRectangular(var l,t,r,b:vec_t;func:planar_func_t;
  zsplits,baseplane:integer):patchprofile_p;stdcall;external V_LIB_DLL
  name 'ppCreateRectangularTHK';
function  ppCreateTriangular(var center:vec3_t;var radius,angle0:vec_t;
  gibbs_abc:vec3_t_array_p;sheffe:spatial_func_t;zsplits:integer;
  baseplane:integer):patchprofile_p;stdcall;external V_LIB_DLL
  name 'ppCreateTriangularTHK';
procedure ppFree(pp:patchprofile_p);stdcall;external V_LIB_DLL;

{   ===================================================================== }
{   u_log.hpp                                                             }
const
  LOG_ANY                               = 0;
  LOG_ERROR                             = $10;
  LOG_WARNING                           = $20;
  LOG_INFO                              = $30;
  LOG_TRACE                             = $40;

type
  con_puts_t                            = procedure(str:pchar);stdcall;
  con_puts_p                            = ^con_puts_t;

function  conAddCallback(outcallback:con_puts_p):Boolean;stdcall;
  external V_LIB_DLL;
procedure conRemoveCallback(outcallback:con_puts_p);stdcall;
  external V_LIB_DLL;
procedure conUpperLevel(level:integer);stdcall;external V_LIB_DLL;
function  conOpenFilelog(name:pchar;flush_every_write:Boolean):Boolean;
  stdcall;external V_LIB_DLL;
procedure conCloseFilelog;stdcall;external V_LIB_DLL;
function  conStartConsolelog:Boolean;stdcall;external V_LIB_DLL;
procedure conStopConsolelog;stdcall;external V_LIB_DLL;
procedure con_puts(str:pchar);stdcall;external V_LIB_DLL;
procedure con_lputs(level:integer;str:pchar);stdcall;external V_LIB_DLL;
procedure con_printf(fmt:pchar;const args:array of const);
procedure con_lprintf(level:integer;fmt:pchar;const args:array of const);
{   ===================================================================== }
{   u_ref2d.hpp                                                           }

type
    idleproc_t=procedure;stdcall;
    graph_settings_p=^graph_settings_t;
    graph_settings_t=record
      cls,title,icon            : pChar;
      sizex,sizey,fullscreen    : integer;
    end;
    gc_t                        = pointer;

function  graphCreateContext(start:graph_settings_p):gc_t;stdcall;
  external V_LIB_DLL;
procedure graphDeleteContext(context:gc_t);stdcall;external V_LIB_DLL;
procedure graphOnDelete(proc:idleproc_t);stdcall;external V_LIB_DLL;
procedure graphMakeCurrent(context:gc_t);stdcall;external V_LIB_DLL;
function  graphGetCurrentContext:gc_t;stdcall;external V_LIB_DLL;
procedure graphFlush(context:gc_t);stdcall;external V_LIB_DLL;
function  graphProcessSysMessages:Boolean;stdcall;external V_LIB_DLL;
function  graphSysMessageloop(OnIdle:idleproc_t):integer;stdcall;
  external V_LIB_DLL;
function  getmaxx:integer;stdcall;external V_LIB_DLL;
function  getmaxy:integer;stdcall;external V_LIB_DLL;
procedure clearviewport;stdcall;external V_LIB_DLL;
procedure moveto(x,y:integer);stdcall;external V_LIB_DLL;
procedure lineto(x,y:integer);stdcall;external V_LIB_DLL;
procedure line(x1,y1,x2,y2:integer);stdcall;external V_LIB_DLL;
procedure rectangle(x1,y1,x2,y2:integer);stdcall;external V_LIB_DLL;
procedure bar(x1,y1,x2,y2:integer);stdcall;external V_LIB_DLL;
procedure circle(x,y,radius:integer);stdcall;external V_LIB_DLL;
procedure ellipse(x1,y1,x2,y2:integer);stdcall;external V_LIB_DLL;
procedure triangle(x1,y1,x2,y2,x3,y3:integer);stdcall;external V_LIB_DLL;
procedure getextent(str:pChar;var sx,sy:integer);stdcall;external V_LIB_DLL;
procedure outtext(str:pChar);stdcall;external V_LIB_DLL;
procedure outtextxy(x,y:integer;str:pChar);stdcall;external V_LIB_DLL;
procedure setcolor(r,g,b:integer);stdcall;external V_LIB_DLL;
procedure setbkcolor(r,g,b:integer);stdcall;external V_LIB_DLL;
procedure settextcolor(r,g,b:integer);stdcall;external V_LIB_DLL;
procedure loadpalette(unix_resname:pChar);stdcall;external V_LIB_DLL;
procedure setindex(index:integer);stdcall;external V_LIB_DLL;
procedure setbkindex(index:integer);stdcall;external V_LIB_DLL;
procedure settextindex(index:integer);stdcall;external V_LIB_DLL;

implementation

{   ===================================================================== }
{   u_x86.cpp                                                             }

function  u_abs(x:vec_t):vec_t;
begin
  Result:=abs(x);
end;

procedure u_fsincos(var s,c:vec_t;r:vec_t);
var
  _s,_c:Extended;
begin
  SinCos(r,_s,_c);
  s:=_s;
  c:=_c;
end;

{   ===================================================================== }
{   u_math.cpp                                                            }

function ToRadians(a:vec_t):vec_t;
begin
  Result:=a*RAD_IN_ANGLE;
end;

function ToDegrees(a:vec_t):vec_t;
begin
  Result:=a*ANGLE_IN_RAD;
end;

procedure vec2Set(var vec:vec2_t;x,y:vec_t);
begin
  vec[0]:=x;
  vec[1]:=y;
end;

procedure vec3Set(var vec:vec3_t;x,y,z:vec_t);
begin
  vec[0]:=x;
  vec[1]:=y;
  vec[2]:=z;
end;

procedure vec4Set(var vec:vec4_t;x,y,z,w:vec_t);
begin
  vec[0]:=x;
  vec[1]:=y;
  vec[2]:=z;
  vec[3]:=w;
end;

procedure vec2Zero(var vec:vec2_t);
begin
  vec[0]:=0.0;
  vec[1]:=0.0;
end;
procedure vec3Zero(var vec:vec3_t);
begin
  vec[0]:=0.0;
  vec[1]:=0.0;
  vec[2]:=0.0;
end;
procedure vec4Zero(var vec:vec4_t);
begin
  vec[0]:=0.0;
  vec[1]:=0.0;
  vec[2]:=0.0;
  vec[3]:=0.0;
end;

procedure vec2Copy(var dst,src:vec2_t);
begin
  dst[0]:=src[0];
  dst[1]:=src[1];
end;

procedure vec3Copy(var dst,src:vec3_t);
begin
  dst[0]:=src[0];
  dst[1]:=src[1];
  dst[2]:=src[2];
end;

procedure vec4Copy(var dst,src:vec4_t);
begin
  dst[0]:=src[0];
  dst[1]:=src[1];
  dst[2]:=src[2];
  dst[3]:=src[3];
end;

procedure vec2Add(var dst,v1,v2:vec2_t);
begin
  dst[0]:=v1[0]+v2[0];
  dst[1]:=v1[1]+v2[1];
end;

procedure vec3Add(var dst,v1,v2:vec3_t);
begin
  dst[0]:=v1[0]+v2[0];
  dst[1]:=v1[1]+v2[1];
  dst[2]:=v1[2]+v2[2];
end;

procedure vec4Add(var dst,v1,v2:vec4_t);
begin
  dst[0]:=v1[0]+v2[0];
  dst[1]:=v1[1]+v2[1];
  dst[2]:=v1[2]+v2[2];
  dst[3]:=v1[3]+v2[3];
end;

procedure vec2Sub(var dst,v1,v2:vec2_t);
begin
  dst[0]:=v1[0]-v2[0];
  dst[1]:=v1[1]-v2[1];
end;

procedure vec3Sub(var dst,v1,v2:vec3_t);
begin
  dst[0]:=v1[0]-v2[0];
  dst[1]:=v1[1]-v2[1];
  dst[2]:=v1[2]-v2[2];
end;

procedure vec4Sub(var dst,v1,v2:vec4_t);
begin
  dst[0]:=v1[0]-v2[0];
  dst[1]:=v1[1]-v2[1];
  dst[2]:=v1[2]-v2[2];
  dst[3]:=v1[3]-v2[3];
end;

procedure vec2Scale(var dst,src:vec2_t;scale:vec_t);
begin
  dst[0]:=src[0]*scale;
  dst[1]:=src[1]*scale;
end;

procedure vec3Scale(var dst,src:vec3_t;scale:vec_t);
begin
  dst[0]:=src[0]*scale;
  dst[1]:=src[1]*scale;
  dst[2]:=src[2]*scale;
end;

procedure vec4Scale(var dst,src:vec4_t;scale:vec_t);
begin
  dst[0]:=src[0]*scale;
  dst[1]:=src[1]*scale;
  dst[2]:=src[2]*scale;
  dst[3]:=src[3]*scale;
end;

procedure vec2Negate(var dst,src:vec2_t);
begin
  dst[0]:=-src[0];
  dst[1]:=-src[1];
end;

procedure vec3Negate(var dst,src:vec3_t);
begin
  dst[0]:=-src[0];
  dst[1]:=-src[1];
  dst[2]:=-src[2];
end;

procedure vec4Negate(var dst,src:vec4_t);
begin
  dst[0]:=-src[0];
  dst[1]:=-src[1];
  dst[2]:=-src[2];
  dst[3]:=-src[3];
end;

procedure vec2NegateSelf(var v:vec2_t);
begin
  v[0]:=-v[0];
  v[1]:=-v[1];
end;

procedure vec3NegateSelf(var v:vec3_t);
begin
  v[0]:=-v[0];
  v[1]:=-v[1];
  v[2]:=-v[2];
end;

procedure vec4NegateSelf(var v:vec4_t);
begin
  v[0]:=-v[0];
  v[1]:=-v[1];
  v[2]:=-v[2];
  v[3]:=-v[3];
end;

function  vec2Dot(var v1,v2:vec2_t):vec_t;
begin
  Result:=v1[0]*v2[0]+v1[1]*v2[1];
end;

function  vec3Dot(var v1,v2:vec3_t):vec_t;
begin
  Result:=v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2];
end;

function  vec4Dot(var v1,v2:vec4_t):vec_t;
begin
  Result:=v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2]+v1[3]*v2[3];
end;

function  vec2Abs(var vec:vec2_t):vec_t;
begin
  Result:=sqrt(vec[0]*vec[0]+vec[1]*vec[1]);
end;

function  vec3Abs(var vec:vec3_t):vec_t;
begin
  Result:=sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]);
end;

function  vec4Abs(var vec:vec4_t):vec_t;
begin
  Result:=sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]+vec[3]*vec[3]);
end;

procedure vec2Unit(var dst,vec:vec2_t);
var
  rabs : vec_t;
begin
  rabs:=1/sqrt(vec[0]*vec[0]+vec[1]*vec[1]);
  vec2Scale(dst,vec,rabs);
end;

procedure vec3Unit(var dst,vec:vec3_t);
var
  rabs : vec_t;
begin
  rabs:=1/sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]);
  vec3Scale(dst,vec,rabs);
end;

procedure vec4Unit(var dst,vec:vec4_t);
var
  rabs : vec_t;
begin
  rabs:=1/sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]+vec[3]*vec[3]);
  vec4Scale(dst,vec,rabs);
end;

procedure vec4Homogenize(var dst,vec:vec4_t);
var
  rw : vec_t;
begin
  rw:=1/vec[3];
  dst[0]:=vec[0]*rw;
  dst[1]:=vec[1]*rw;
  dst[2]:=vec[2]*rw;
  dst[3]:=1.0;
end;

function  vec2Cross(var v1,v2:vec2_t):vec_t;
begin
  Result:=v1[0]*v2[1]-v1[1]*v2[0];
end;

function  vec3Mix(var v1,v2,v3:vec3_t):vec_t;
var
  v : vec3_t;
begin
  vec3Cross(v,v1,v2);
  Result:=vec3Dot(v,v3);
end;

function  vec2MinInd(var vec:vec2_t):integer;
var
  l1,l2 : vec_t;
begin
  l1:=Abs(vec[0]);
  l2:=Abs(vec[1]);
  if l1>l2 then Result:=1 else Result:=0;
end;

function  vec2MaxInd(var vec:vec2_t):integer;
var
  l1,l2 : vec_t;
begin
  l1:=Abs(vec[0]);
  l2:=Abs(vec[1]);
  if l1<l2 then Result:=1 else Result:=0;
end;

function  vec3MinInd(var vec:vec3_t):integer;
var
  l0,l1,l2 : vec_t;
begin
  l0:=Abs(vec[0]);
  l1:=Abs(vec[1]);
  l2:=Abs(vec[2]);
  if l0<l1 then begin
    if l0<l2 then
      Result:=0 
    else 
      Result:=2;
  end else begin
    if l1<l2 then
      Result:=1
    else 
      Result:=2;
  end;
end;

function  vec3MaxInd(var vec:vec3_t):integer;
var
  l0,l1,l2 : vec_t;
begin
  l0:=Abs(vec[0]);
  l1:=Abs(vec[1]);
  l2:=Abs(vec[2]);
  if l0>l1 then begin
    if l0>l2 then
      Result:=0 
    else 
      Result:=2;
  end else begin
    if l1>l2 then
      Result:=1
    else 
      Result:=2;
  end;
end;

function  vec4MinInd(var vec:vec4_t):integer;
var
  l,min : vec_t;
  i,ret : integer;
begin
  ret:=0;
  min:=Abs(vec[ret]);
  for i:=1 to 3 do begin
    l:=Abs(vec[i]);
    if l<min then begin
      min:=l;
      ret:=i;
    end;
  end;
  Result:=ret;
end;

function  vec4MaxInd(var vec:vec4_t):integer;
var
  l,max : vec_t;
  i,ret : integer;
begin
  ret:=0;
  max:=Abs(vec[ret]);
  for i:=1 to 3 do begin
    l:=Abs(vec[i]);
    if l>max then begin
      max:=l;
      ret:=i;
    end;
  end;
  Result:=ret;
end;

function  IsZero(v:vec_t):Boolean;
begin
  Result:=Abs(v)<FLOAT_ZERO_EPSILON;
end;

function  vec2IsZero(var vec:vec2_t):Boolean;
begin
  Result:=IsZero(vec[0]) and IsZero(vec[1]);
end;

function  vec3IsZero(var vec:vec3_t):Boolean;
begin
  Result:=IsZero(vec[0]) and IsZero(vec[1]) and IsZero(vec[2]);
end;

function  vec4IsZero(var vec:vec4_t):Boolean;
begin
  Result:=IsZero(vec[0]) and IsZero(vec[1]) and IsZero(vec[2])
                         and IsZero(vec[3]);
end;

function  vec2IsEqual(var v1,v2:vec2_t):Boolean;
begin
  Result:=IsZero(v1[0]-v2[0]) and IsZero(v1[1]-v2[1]);
end;

function  vec3IsEqual(var v1,v2:vec3_t):Boolean;
begin
  Result:=IsZero(v1[0]-v2[0]) and IsZero(v1[1]-v2[1]) and IsZero(v1[2]-v2[2]);
end;

function  vec4IsEqual(var v1,v2:vec4_t):Boolean;
begin
  Result:=IsZero(v1[0]-v2[0]) and IsZero(v1[1]-v2[1])
    and IsZero(v1[2]-v2[2]) and IsZero(v1[3]-v2[3]);
end;

function  vec2IsEqualEpsilon(var a,b:vec2_t;epsilon:vec_t):Boolean;
var
  dif : vec2_t;
  dis : vec_t;
begin
  vec2Sub(dif,a,b);
  dis:=vec2Abs(dif);
  Result:=dis < epsilon;
end;

function  vec3IsEqualEpsilon(var a,b:vec3_t;epsilon:vec_t):Boolean;
var
  dif : vec3_t;
  dis : vec_t;
begin
  vec3Sub(dif,a,b);
  dis:=vec3Abs(dif);
  Result:=dis < epsilon;
end;

function  vec4IsEqualEpsilon(var a,b:vec4_t;epsilon:vec_t):Boolean;
var
  dif : vec4_t;
  dis : vec_t;
begin
  vec4Sub(dif,a,b);
  dis:=vec4Abs(dif);
  Result:=dis < epsilon;
end;

procedure vec2ScaleByVec2(var dst,v1,v2:vec2_t);
begin
  dst[0]:=v1[0]*v2[0];
  dst[1]:=v1[1]*v2[1];
end;

procedure vec3ScaleByVec3(var dst,v1,v2:vec3_t);
begin
  dst[0]:=v1[0]*v2[0];
  dst[1]:=v1[1]*v2[1];
  dst[2]:=v1[2]*v2[2];
end;

procedure vec4ScaleByVec4(var dst,v1,v2:vec4_t);
begin
  dst[0]:=v1[0]*v2[0];
  dst[1]:=v1[1]*v2[1];
  dst[2]:=v1[2]*v2[2];
  dst[3]:=v1[3]*v2[3];
end;

procedure vec2ZoomIF(var zoom,initial,final:vec2_t);
begin
  zoom[0]:=final[0]/initial[0];
  zoom[1]:=final[1]/initial[1];
end;

procedure vec3ZoomIF(var zoom,initial,final:vec3_t);
begin
  zoom[0]:=final[0]/initial[0];
  zoom[1]:=final[1]/initial[1];
  zoom[2]:=final[2]/initial[2];
end;

procedure vec4ZoomIF(var zoom,initial,final:vec4_t);
begin
  zoom[0]:=final[0]/initial[0];
  zoom[1]:=final[1]/initial[1];
  zoom[2]:=final[2]/initial[2];
  zoom[3]:=final[3]/initial[3];
end;

procedure vec2ToRadians(var dst,src:vec2_t);
begin
  vec2Scale(dst,src,ANGLE_TO_RAD);
end;

procedure vec3ToRadians(var dst,src:vec3_t);
begin
  vec3Scale(dst,src,ANGLE_TO_RAD);
end;

procedure vec4ToRadians(var dst,src:vec4_t);
begin
  vec4Scale(dst,src,ANGLE_TO_RAD);
end;

procedure vec2ToDegrees(var dst,src:vec2_t);
begin
  vec2Scale(dst,src,RAD_TO_ANGLE);
end;

procedure vec3ToDegrees(var dst,src:vec3_t);
begin
  vec3Scale(dst,src,RAD_TO_ANGLE);
end;

procedure vec4ToDegrees(var dst,src:vec4_t);
begin
  vec4Scale(dst,src,RAD_TO_ANGLE);
end;

procedure mat2Zero(var mat:matrix2_t);
begin
  u_zerodw(@mat,sizeof(vec_t));
end;

procedure mat3Zero(var mat:matrix3_t);
begin
  u_zerodw(@mat,9 * sizeof(vec_t) shr 2);
end;

procedure mat4Zero(var mat:matrix4_t);
begin
  u_zerodw(@mat,sizeof(vec_t) shl 2);
end;

procedure mat3x4Zero(var mat:matrix3x4_t);
begin
  u_zerodw(@mat,3*sizeof(vec_t));
end;

procedure mat2Copy(var dst,src:matrix2_t);
begin
  u_copydw(@dst,@src,sizeof(vec_t));
end;

procedure mat3Copy(var dst,src:matrix3_t);
begin
  u_copydw(@dst,@src,9 * sizeof(vec_t) shr 2);
end;

procedure mat4Copy(var dst,src:matrix4_t);
begin
  u_copydw(@dst,@src,sizeof(vec_t) shl 2);
end;

procedure mat3x4Copy(var dst,src:matrix3x4_t);
begin
  u_copydw(@dst,@src,3*sizeof(vec_t));
end;

procedure mat2Unit(var mat:matrix2_t);
begin
  mat[1][0]:=0.0;
  mat[0][1]:=0.0;
  mat[0][0]:=1.0;
  mat[1][1]:=1.0;
end;

procedure mat3Unit(var mat:matrix3_t);
begin
  mat3Zero(mat);
  mat[0][0]:=1.0;
  mat[1][1]:=1.0;
  mat[2][2]:=1.0;
end;

procedure mat4Unit(var mat:matrix4_t);
begin
  mat4Zero(mat);
  mat[0][0]:=1.0;
  mat[1][1]:=1.0;
  mat[2][2]:=1.0;
  mat[3][3]:=1.0;
end;

procedure mat3x4Unit(var mat:matrix3x4_t);
begin
  mat3x4Zero(mat);
  mat[0][0]:=1.0;
  mat[1][1]:=1.0;
  mat[2][2]:=1.0;
end;

procedure mat2RowFromVec2(var mat:matrix2_t;row:integer;var vec:vec2_t);
begin
  mat[row][0]:=vec[0];
  mat[row][1]:=vec[1];
end;
procedure mat2Transpose(var dst,src:matrix2_t);
begin
  dst[0][0]:=src[0][0];
  dst[0][1]:=src[1][0];
  dst[1][0]:=src[0][1];
  dst[1][1]:=src[1][1];
end;

procedure mat2TransposeSelf(var mat:matrix2_t);
var
  temp  : vec_t;
begin
  temp:=mat[0][1];mat[0][1]:=mat[1][0];mat[1][0]:=temp;
end;

procedure vec3Transform(var vec:vec3_t;var mat:matrix3x4_t);
var
  temp  : vec3_t;
begin
  vec3Copy(temp,vec);
  mat3x4ByVec3(vec,mat,temp);
end;

procedure mat3x4ConcatBefore(var dst,transform:matrix3x4_t);
var
  tmp : matrix3x4_t;
begin
  mat3x4ByMat3x4(tmp,transform,dst);
  mat3x4Copy(dst,tmp);
end;

procedure mat3x4ConcatAfter(var dst,transform:matrix3x4_t);
var
  tmp : matrix3x4_t;
begin
  mat3x4ByMat3x4(tmp,dst,transform);
  mat3x4Copy(dst,tmp);
end;

procedure rgbScalarFromByte(var rgb:vec3_t;var brgb:bytergb_t);
begin
  rgb[0]:=brgb[0]/255.0;
  rgb[1]:=brgb[1]/255.0;
  rgb[2]:=brgb[2]/255.0;
end;

procedure rgbaScalarFromByte(var rgba:vec4_t;var brgba:bytergba_t);
begin
  rgba[0]:=brgba[0]/255.0;
  rgba[1]:=brgba[1]/255.0;
  rgba[2]:=brgba[2]/255.0;
  rgba[3]:=brgba[3]/255.0;
end;

procedure rgbByteFromScalar(var brgb:bytergb_t;var rgb:vec3_t);
begin
  brgb[0]:=round(rgb[0]*255);
  brgb[1]:=round(rgb[1]*255);
  brgb[2]:=round(rgb[2]*255);
end;

procedure rgbaByteFromScalar(var brgba:bytergba_t;var rgba:vec4_t);
begin
  brgba[0]:=round(rgba[0]*255);
  brgba[1]:=round(rgba[1]*255);
  brgba[2]:=round(rgba[2]*255);
  brgba[3]:=round(rgba[3]*255);
end;

{   ===================================================================== }
{   u_vector.hpp                                                          }

function vecAlloc(size:integer):vector_p;
var
  res : vector_p;
begin
  GetMem(pointer(res),sizeof(integer)+size*sizeof(vec_t));
  res^.size:=size;
  Result:=res;
end;

procedure vecCopy(dst,src:vector_p);
begin
  u_copydw(dst^.elements,src^.elements,dst^.size*sizeof(vec_t) shr 2);
end;

procedure vecFree(v:vector_p);
begin
  FreeMem(pointer(v));
end;

function vecDuplicate(v:vector_p;keepon:Boolean):vector_p;
var
  v2 : vector_p;
begin
  v2:=vecAlloc(v^.size);
  vecCopy(v2,v);
  if not keepon then vecFree(v);
  Result:=v2;
end;

procedure vecZero(v:vector_p);
begin
  u_zerodw(v^.elements,v^.size*sizeof(vec_t) shr 2);
end;

function vecDot(v1,v2:vector_p):vec_t;
var
  i : integer;
  r : vec_t;
begin
  r:=0;
  i:=0;
  while i < v1^.size do begin
    r:=r+v1^.elements[i]+v2^.elements[i];
    i:=i+1;
  end;
  Result:=r;
end;

function vecAbs(v:vector_p):vec_t;
begin
  Result:=sqrt(vecDot(v,v));
end;

procedure vecScale(dest,v:vector_p;x:vec_t);
var
  i : integer;
begin
  i:=0;
  while i<v^.size do begin
    dest^.elements[i]:=v^.elements[i]*x;
    i:=i+1;
  end;
end;

function vecUnit(v:vector_p):Boolean;
var
  ra : vec_t;
begin
  ra:=vecAbs(v);
  if IsZero(ra) then begin
    Result:=False;
    exit;
  end;
  ra:=1.0/ra;
  vecScale(v,v,ra);
  Result:=True;
end;

{   ===================================================================== }
{   u_matrix.cpp                                                          }

procedure matCopy(dest,src:matrix_p);
begin
  u_copydw(dest^.cell,src^.cell,src^.rows*sizeof(vec_p) shr 2);
  u_copydw(dest^.data,src^.data,src^.rows*src^.cols*sizeof(vec_p) shr 2);
end;

procedure matZero(m:matrix_p);
begin
  u_zerodw(m^.data,m^.rows*m^.cols*sizeof(vec_t) shr 2);
end;

procedure matDetermTHK(m:matrix_p;keepon:Boolean;var res:vec_t);stdcall;
  external V_LIB_DLL;

function  matDeterm(m:matrix_p;keepon:Boolean):vec_t;
var
  res : vec_t;
begin
  matDetermTHK(m,keepon,res);
  Result:=res;
end;

procedure matScale(dest,m:matrix_p;x:vec_t);
var
  i,j : integer;
begin
  i:=0;
  j:=0;
  while i<m^.rows do begin
    while j<m^.cols do begin
      dest^.cell^[i]^[j]:=m^.cell^[i]^[j]*x;
      j:=j+1;
    end;
    i:=i+1;
  end;
end;

{   ===================================================================== }
{   u_plane.cpp                                                           }

procedure poly2AddPoint(w:poly2_p;var pt:vec2_t);
begin
  vec2Copy(w^.rays^[w^.count].origin,pt);
  w^.count:=w^.count+1;
end;

function  distFromLine2(var p:vec2_t;line:line2_p):vec_t;
begin
  Result:=vec2Dot(p,line^.normal)+line^.dist;
end;

procedure mat2Rotation(var mat:matrix2_t;radians:vec_t);
var
  s, c : Extended;
begin
  SinCos(radians,s,c);
  mat[0][1]:=s;
  mat[1][0]:=-mat[0][1];
  mat[0][0]:=c;
  mat[1][1]:=mat[0][0];
end;

{   ===================================================================== }
{   u_space.cpp                                                           }

function  distFromPlane3(var pt:vec3_t;pl:plane3_p):vec_t;
begin
  Result:=vec3Dot(pl^.normal,pt) + pl^.dist;
end;

{   ===================================================================== }
{   u_log.cpp                                                             }

procedure con_printf(fmt:pchar;const args:array of const);
var
  s : string;
begin
  s:=Format(fmt,args);
  con_puts(pchar(s));
end;

procedure con_lprintf(level:integer;fmt:pchar;const args:array of const);
var
  s : string;
begin
  s:=Format(fmt,args);
  con_lputs(level,pchar(s));
end;

{   ===================================================================== }

end.

//</PRE>
//</HTML>

